package sociogram;
import java.awt.Color;
import java.awt.Graphics;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import simulation.*;
import java.util.*;
import javax.swing.*;

/**
 * Graphical User Interface (GUI) for performing simulations. It implements
 * ObserverAnimation for being updated each simulation finishes, for starting 
 * with the next one.
 * @author Iván García-Magariño
 */
public class MainGUI extends javax.swing.JFrame implements ObserverAnimation {

    /** Current simulation **/
    private SimulationAnimation simulation;
    
    /** Reference to the Sociogram GUI **/
    SociogramGUI sociogramGUI=null;
    
    /** Reference to the Measurement Evolution of Sociograms **/
    MeasurementEvolutionSociogram measurement;
    
    /** Measurement evolutions of each simulation **/
    private MeasurementEvolutionSociogram[] measurements;
    /** Final sociogram of each simulation */
    private Sociogram[] sociograms;
    /** Number of Simulations **/
    private int numSimulations;
    /** Counter for the number of simulation that is being run **/
    private int contSim;
    /** Matrix of NxM values where N is the number of simulations and 
     * M is the number of metrics. Each row has the metrics in the following
     order: (IAg, ICg, IDg, IIg).
     This matrix is used for calculating the averages and standard deviations*/
    private double [][] results;
    /** This matrix stores the summarizes  of the results for each metric. 
     * It uses the following summarizes in this order in each row:
     * - Avaerages
     * - Standard Deviations
     *  The matrix couldbe represented as summary[summarizeIndex][metricIndex]
     */
    private double [][]summary;
    
   
    /** File Chooser for saving the results */
    private JFileChooser chooser;
    
    
    /** Num of summaries (for now 2, Averages and Standard Deviatons */
    public static int NUM_SUMMARIES=2;
    /** Encoding **/
    public final static Charset ENCODING=StandardCharsets.ISO_8859_1;
    /** Column Length for saving several simulations */
    public final static int COLUMN_LENGTH=15;
    
    /**
     * Creates new form MainGUI
     */
    public MainGUI() {
        initComponents();
        initOtherComponents(); // manually programmed
    }
    

    
   
    
    /** Performs a simulation according to the data **/
    private void performSimulation(){
        
        // Creates the simulation animation and register as an observer
        simulation = new SimulationAnimation(this);
        simulation.setObserver(this);
        Blackboard.getInstance().setSimulation(simulation);
        try{
            // Creates the agents and register these
            createAgents();
            // Set the speed of the simulation
            double speed=readSpeed();
            simulation.setSpeed(speed);
            // Get the number of iterations
            int numIterationsClass=readIterations();
            // The simulation is performed in one single step (i.e. iteration)
            // to reduce the variability according to guidelines of deck random generators
            int numIterations=1;
            // Set a measurement evolution objects to record the evolution
            // of sociometrics
            measurement = new MeasurementEvolutionSociogram(numIterations);
            simulation.setMeasurementEvolution(measurement);
            // Run the simulation with the number of iterations
            simulation.run(numIterations);
            // Show the results in graphs
            showResults();
        /* In case some numeric field is empty or without a 
        number, the user is warned */
        }catch(NumberFormatException e){
            JOptionPane.showMessageDialog(this,
                "The inputs of the simulation must contain "
                    + "numeric data. \n "
                    + "Please check that the corresponding "
                    + "fields only contain numbers.");
        }
    }
    /** Creates the agents and register these in the 
     * simulation
     * */
    private void createAgents(){
        // Create an empty list of agents
        List<IndividualAgent> list=new LinkedList<IndividualAgent>();
        IndividualAgent agent;
        // Creates agents of each type and add these to the sociogram and simulation
        int numAgents;
        numAgents=Integer.parseInt(this.jTextFieldQuite.getText());
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.QUIET);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=Integer.parseInt(this.jTextFieldParticipant.getText());
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.PARTICIPANT);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=Integer.parseInt(this.jTextFieldTangent.getText());
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.TANGENT);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=Integer.parseInt(this.jTextFieldJoker.getText());
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.JOKER);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=Integer.parseInt(this.jTextFieldObstructive.getText());
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.OBSTRUCTIVE);
            simulation.register(agent);
            list.add(agent);
        }
        numAgents=Integer.parseInt(this.jTextFieldOccasional.getText());
        for(int i=0; i<numAgents; i++){
            agent=new IndividualAgent(AgentType.OCCASIONAL);
            simulation.register(agent);
            list.add(agent);
        }
        // Creates a sociogram from the list of agents
        Blackboard.getInstance().createSociogram(list);
    }
    
    /** Shows the results of the simulation. In this case, it starts a new
     frame with the sociogram. **/
    private void showResults(){
       
    }
    
    /** Returns the number of iterations indicated by the user **/
    private int readIterations(){
        return Integer.parseInt(this.jTextFieldIterations.getText());
    }
    
    /** Returns the number of simulations indicated by the user **/
    private int readSimulations(){
        return Integer.parseInt(this.jTextFieldNumSimulations.getText());
    }
    
    /** Returns the speed (iterations/second) of the simulation indicated 
     * by the user **/
    private double readSpeed(){
        String str=(String) this.jComboBoxSpeed.getSelectedItem();
        double speed=Double.parseDouble(str);
        return speed;     
    }
 
    /** Init other components manually indtroduced by the programmer */
    public void initOtherComponents(){
        chooser= new JFileChooser();
        // Make invisible number iterations for now
        this.jLabelIterations.setVisible(false);
        this.jTextFieldIterations.setVisible(false);
    }
    
    
    
    /** Starts the running of several simulations, and the end of each
     * simulation is notified through the undateFinishedSimulation method."
     */
    public void performSeveralSimulations(){
        // Obtain the number of metrics and init the arrays
        int numMetrics=MeasurementEvolutionSociogram.NUM_METRICS;
        numSimulations=readSimulations();
        sociograms=new Sociogram[numSimulations];
        measurements=new MeasurementEvolutionSociogram[numSimulations];
        results= new double[numSimulations][numMetrics];
        summary= new double[NUM_SUMMARIES][numMetrics];
        // Creates the GUI of the Sociogram
        sociogramGUI= new SociogramGUI();
        sociogramGUI.setVisible(true);
        Blackboard.getInstance().setSociogramGUI(sociogramGUI);
        // Sets the size of the Table of Results in GUI and clear the table of
        // the summary
        changeSizeJTableResults(numSimulations);
        clearJTableSummary();
        // Perform the first simulation.
        contSim=0;   
        performSimulation();
        
    }
    
    /** When a simulation is finished (notified as an observer), it records
     * the state of the finished simulation, and starts the next simulation
     * if necessary.
     */
    public void updateFinishedSimulation() {
        // Record the state
        Sociogram sociogram = Blackboard.getInstance().getSociogram();
        sociograms[contSim]=sociogram;
        measurements[contSim]=measurement; 
        // Store and print the measurement values in the table of the results
        double iag, icg, idg, iig;
        iag=sociogram.measureIAg();
        icg=sociogram.measureICg();
        idg=sociogram.measureIDg();
        iig=sociogram.measureIIg();
        results[contSim][0]=iag;
        results[contSim][1]=icg;
        results[contSim][2]=idg;
        results[contSim][3]=iig;        
        jTableResults.getModel().setValueAt(""+(contSim+1), contSim, 0);
        jTableResults.getModel().setValueAt(Sociogram.METRIC_FORMAT.format(iag), contSim, 1);
        jTableResults.getModel().setValueAt(Sociogram.METRIC_FORMAT.format(icg), contSim, 2);
        jTableResults.getModel().setValueAt(Sociogram.METRIC_FORMAT.format(idg), contSim, 3);
        jTableResults.getModel().setValueAt(Sociogram.METRIC_FORMAT.format(iig), contSim, 4);
        // Increments the counter of simulations
        contSim++;
        // If below the limit, it starts a new simulation
        if(contSim<numSimulations){
            performSimulation();
        }
        // If all the simulations are performed, it calcculates the summary
        // and shows it
        else{
            calculateSummary();
            showSummary();
        }
       
    }
    
    /** Calculates the summary of the results (averages and standard 
     * deviations **/
    public void calculateSummary(){
        int numMetrics=MeasurementEvolutionSociogram.NUM_METRICS;
        ///// Calculates the average of all metrics (all in row zero of matrix)
        int indexAvrg=0;
        // Starts with zero
        for(int i=0; i<numMetrics;i++){
            summary[indexAvrg][i]=0;
        }
        // Gets the sum of results
        for (int i=0;i<results.length;i++){
            for(int j=0;j<numMetrics;j++){
                summary[indexAvrg][j]= summary[indexAvrg][j]+results[i][j];
            }
        }
        // Divides the summary by the number os simulations
        for(int j=0;j<numMetrics;j++){
                summary[indexAvrg][j]= summary[indexAvrg][j]/numSimulations;
        }
        
        ///// Calculates the Standar Deviations of all metrics 
        //(all in row one of matrix)
        int indexSD=1;
        // Starts with zero
        for(int i=0; i<numMetrics;i++){
            summary[indexSD][i]=0;
        }
        // Gets the sum of the squares of the differences
        for (int i=0;i<results.length;i++){
            for(int j=0;j<numMetrics;j++){
                // Difference to the average
                double diff= results[i][j]-summary[indexAvrg][j];
                // Square of the difference
                double square = Math.pow(diff, 2);
                // Sum the square
                summary[indexSD][j]=summary[indexSD][j]+square; 
            }
        }
        // Gets the averages of squares, and then calculate the square root
        for(int j=0;j<numMetrics;j++){
                // Average of squares
                summary[indexSD][j]= summary[indexSD][j]/numSimulations;
                // Calculate the square root
                summary[indexSD][j]= Math.sqrt(summary[indexSD][j]);
        }
        
    }
    /** It shows the summary in the GUI **/
    public void showSummary(){
        // Prints the row headers
        jTableSummary.setValueAt("Average", 0, 0);
        jTableSummary.setValueAt("SD", 1, 0);
        // Copy the summary values
        for (int i=0;i<NUM_SUMMARIES;i++){
            for(int j=0;j<MeasurementEvolutionSociogram.NUM_METRICS;j++){
                // Copy values but but summing one to the column to consider row headers
                jTableSummary.setValueAt(Sociogram.METRIC_FORMAT.format(summary[i][j]), i, j+1);
            }
        }
    }
    
    /** It saves several simulations */
    public void saveSeveralSimulations(PrintStream out){
        out.println("-- Results provided by the ABS-SOCI simulator --");
        // Save the input parameters
        saveInputParameters(out);
        // Save the Results
        out.println();
        out.println("---------- SOCIOMETRICS OF ALL SIMULATIONS ----------");
        saveJTable(jTableResults, out, numSimulations);
        // Save the Summary of the Results
        out.println();
        out.println("---------- SUMMARY OF THE RESULTS ------------");
        saveJTable(jTableSummary, out, NUM_SUMMARIES);
        // Save each simulation (final sociogram and measurement evolution
        for(int i=0;i<numSimulations;i++){
            out.println();
            out.println("----------- SIMULATION "+(i+1)+" ------------");        
             // Save the sociogram
             sociograms[i].save(out);
            // Save the evolution
            measurements[i].saveValues(out);
        }
        
    }
    /** It saves the contents of the JTable. It is used for saving several 
     tables.
     @param jtable The JTable of the GUI that will be saved
     @param out The PrintStream in which the content of the table will be printed.
     @param numRows The number of rows that will be printed**/
    public void saveJTable(JTable jtable, PrintStream out, int numRows){
        String str;
        // Prints the headers
        for(int i=0;i<jtable.getModel().getColumnCount();i++){
            str=jtable.getModel().getColumnName(i);
            str=String.format("%1$"+COLUMN_LENGTH+"s", str);
            out.print(str+"\t");
        }    
        // Print the values
        out.println();
        for (int i=0;i<numRows;i++){
            for(int j=0;j<jtable.getModel().getColumnCount();j++){
                str=(String) jtable.getValueAt(i, j);
                str=String.format("%1$"+COLUMN_LENGTH+"s", str);
                out.print(str+"\t");
            }
            out.println();
        }
    }
    
    /* Save the input parameters from the GUI into the print 
    stream of the parameter */
    public void saveInputParameters(PrintStream out){
        out.println();
        out.println("INPUT PARAMETERS");
        out.println("Quiet Students = "+this.jTextFieldQuite.getText());
        out.println("Participant Students = "+this.jTextFieldParticipant.getText());
        out.println("Tangent Students = "+this.jTextFieldTangent.getText());
        out.println("Joker Students = "+this.jTextFieldJoker.getText());
        out.println("Obstructive Students = "+this.jTextFieldObstructive.getText());
        out.println("Occasional Students = "+this.jTextFieldOccasional.getText());
        out.println("Number of Iterations = "+this.jTextFieldIterations.getText());
        out.println("Number of Simulations = "+this.jTextFieldNumSimulations.getText());        
    }
    
    
    /** It changes the number of rows of jTableResults, so users can define
     * an unlimited number of simulations.
     * @param numRows Number of Rows
     */
    public void changeSizeJTableResults(int numRows){
        String[] titles = new String [] {
                "Id. Simulation", "IAg", "ICg", "IDg", "IIg"
            };
        Object[][] objects=new Object[numRows][titles.length];
        for (int i=0;i<numRows;i++){
            for(int j=0;j<titles.length;j++){
                objects[i][j]=null;
            }
        }
        jTableResults.setModel(new javax.swing.table.DefaultTableModel(
            objects, titles
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane1.setViewportView(jTableResults);
    }
    /** It clears the table with the summaries **/
    public void clearJTableSummary(){
        // Read the number of metrics
        int numMetrics=MeasurementEvolutionSociogram.NUM_METRICS;
        // The number of colums is the the number of metrics plus one
        for (int i=0;i<NUM_SUMMARIES;i++){
            for(int j=0; j<numMetrics+1; j++){
                jTableSummary.setValueAt("", i, j);
            }
        }
    }
    
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        jLabel2 = new javax.swing.JLabel();
        jTextFieldQuite = new javax.swing.JTextField();
        jLabelCouple = new javax.swing.JLabel();
        jTextFieldParticipant = new javax.swing.JTextField();
        jLabelFamilyBabies = new javax.swing.JLabel();
        jTextFieldTangent = new javax.swing.JTextField();
        jLabelFamilyWithoutBabies = new javax.swing.JLabel();
        jTextFieldJoker = new javax.swing.JTextField();
        jLabelIterations = new javax.swing.JLabel();
        jTextFieldIterations = new javax.swing.JTextField();
        jLabelInput1 = new javax.swing.JLabel();
        jButtonRunSimulation = new javax.swing.JButton();
        jLabelFriends = new javax.swing.JLabel();
        jTextFieldObstructive = new javax.swing.JTextField();
        jTextFieldOccasional = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jComboBoxSpeed = new javax.swing.JComboBox();
        jButtonSaveResults = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jTextFieldNumSimulations = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTableResults = new javax.swing.JTable();
        jScrollPane3 = new javax.swing.JScrollPane();
        jTableSummary = new javax.swing.JTable();
        jLabel7 = new javax.swing.JLabel();

        jLabel1.setText("jLabel1");

        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane2.setViewportView(jTable1);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("ABS-SOCI (with Deck random generators)");

        jLabel2.setText("Quiet Students:");

        jTextFieldQuite.setText("0");
        jTextFieldQuite.setToolTipText("");
        jTextFieldQuite.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldQuiteActionPerformed(evt);
            }
        });

        jLabelCouple.setText("Participant Students:");

        jTextFieldParticipant.setText("100");
        jTextFieldParticipant.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldParticipantActionPerformed(evt);
            }
        });

        jLabelFamilyBabies.setText("Tangent Students:");

        jTextFieldTangent.setText("0");
        jTextFieldTangent.setToolTipText("");
        jTextFieldTangent.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldTangentActionPerformed(evt);
            }
        });

        jLabelFamilyWithoutBabies.setText("Joker Students:");

        jTextFieldJoker.setText("0");
        jTextFieldJoker.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldJokerActionPerformed(evt);
            }
        });

        jLabelIterations.setText("Number of Iterations:");

        jTextFieldIterations.setText("1");
        jTextFieldIterations.setToolTipText("");
        jTextFieldIterations.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldIterationsActionPerformed(evt);
            }
        });

        jLabelInput1.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        jLabelInput1.setText("Input of Simulations");
        jLabelInput1.setToolTipText("");

        jButtonRunSimulation.setText("Run Simulations");
        jButtonRunSimulation.setToolTipText("");
        jButtonRunSimulation.setActionCommand("RunSimulation");
        jButtonRunSimulation.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runSimulationActionPerformed(evt);
            }
        });

        jLabelFriends.setText("Obstructive Students:");

        jTextFieldObstructive.setText("0");
        jTextFieldObstructive.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldObstructiveActionPerformed(evt);
            }
        });

        jTextFieldOccasional.setText("0");
        jTextFieldOccasional.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldOccasionalActionPerformed(evt);
            }
        });

        jLabel3.setText("Occasional Participant Students:");

        jLabel4.setText("Iterations / second:");

        jComboBoxSpeed.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "0.5", "1.3", "2", "8", "20", "50", "200" }));
        jComboBoxSpeed.setSelectedIndex(6);
        jComboBoxSpeed.setToolTipText("");
        jComboBoxSpeed.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxSpeedActionPerformed(evt);
            }
        });

        jButtonSaveResults.setText("Save Results");
        jButtonSaveResults.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSaveResultsActionPerformed(evt);
            }
        });

        jLabel5.setText("Number of Simulations:");

        jTextFieldNumSimulations.setText("10");
        jTextFieldNumSimulations.setToolTipText("");
        jTextFieldNumSimulations.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldNumSimulationsActionPerformed(evt);
            }
        });

        jLabel6.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        jLabel6.setText("Results");

        jTableResults.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null}
            },
            new String [] {
                "Id. Simulation", "IAg", "ICg", "IDg", "IIg"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane1.setViewportView(jTableResults);

        jTableSummary.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null},
                {null, null, null, null, null}
            },
            new String [] {
                "", "IAg", "ICg", "IDg", "IIg"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.Object.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, true, true
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane3.setViewportView(jTableSummary);

        jLabel7.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        jLabel7.setText("Summary of the Results");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(19, 19, 19)
                        .addComponent(jLabelInput1))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jButtonRunSimulation)
                        .addGap(18, 18, 18)
                        .addComponent(jButtonSaveResults))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jLabel2)
                                .addComponent(jLabelCouple, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jLabelFamilyBabies)
                                .addComponent(jLabelFamilyWithoutBabies)
                                .addComponent(jLabel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jLabelFriends)
                                .addComponent(jLabelIterations))
                            .addComponent(jLabel4)
                            .addComponent(jLabel5))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jTextFieldNumSimulations, javax.swing.GroupLayout.PREFERRED_SIZE, 55, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jComboBoxSpeed, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                    .addComponent(jTextFieldObstructive, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jTextFieldParticipant, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jTextFieldQuite, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jTextFieldTangent, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jTextFieldJoker, javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jTextFieldOccasional, javax.swing.GroupLayout.PREFERRED_SIZE, 53, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addComponent(jTextFieldIterations, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 53, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                .addGap(42, 42, 42)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel7)
                    .addComponent(jLabel6)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 452, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(25, 25, 25))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(21, 21, 21)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabelInput1)
                    .addComponent(jLabel6))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel2)
                            .addComponent(jTextFieldQuite, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelCouple)
                            .addComponent(jTextFieldParticipant))
                        .addGap(4, 4, 4)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelFamilyBabies)
                            .addComponent(jTextFieldTangent))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelFamilyWithoutBabies)
                            .addComponent(jTextFieldJoker))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelFriends)
                            .addComponent(jTextFieldObstructive))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jTextFieldOccasional, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel3))
                        .addGap(14, 14, 14)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelIterations)
                            .addComponent(jTextFieldIterations)))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 192, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(jLabel4)
                    .addComponent(jComboBoxSpeed, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(8, 8, 8)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel5)
                            .addComponent(jTextFieldNumSimulations, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jButtonRunSimulation)
                            .addComponent(jButtonSaveResults)))
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 93, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(35, 35, 35))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jTextFieldQuiteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldQuiteActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldQuiteActionPerformed

    private void jTextFieldParticipantActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldParticipantActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldParticipantActionPerformed

    private void jTextFieldTangentActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldTangentActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldTangentActionPerformed

    private void jTextFieldJokerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldJokerActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldJokerActionPerformed

    private void jTextFieldIterationsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldIterationsActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldIterationsActionPerformed
    /** Method for running the simulations, executed when pressing the "Run 
     * Simulation" button.
     * @param evt 
     */
    private void runSimulationActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runSimulationActionPerformed
        performSeveralSimulations();
            
    }//GEN-LAST:event_runSimulationActionPerformed

    private void jTextFieldObstructiveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldObstructiveActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldObstructiveActionPerformed
    
    private void jTextFieldOccasionalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldOccasionalActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldOccasionalActionPerformed
    
    
    /* When pressing the "Save Results" button, the results are save alongside
    the inpu parameters*/
    private void jButtonSaveResultsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSaveResultsActionPerformed
        // Checks whether a sociogram exists 
        if(Blackboard.getInstance().getSociogram()==null){
            JOptionPane.showMessageDialog(this,"Please, run a simulation "
                    + "before this command.");
            return;
        } 
        // Let the user select the file
        int returnValue=chooser.showSaveDialog(this);
        if (returnValue == JFileChooser.APPROVE_OPTION){
            // Get the File and its path
            File file=chooser.getSelectedFile();
            String strPath=file.getAbsolutePath();
            Path path = Paths.get(strPath);
            // Open the File for Writing on it
            try{
                PrintStream out= new PrintStream(
                        new FileOutputStream(strPath));
                // Save several Simulations
                saveSeveralSimulations(out);
                // Close the file
                out.close();
            }catch(FileNotFoundException e){
                System.out.println("Error when saving file "+
                        file.getName());
            }
        }
        
    }//GEN-LAST:event_jButtonSaveResultsActionPerformed

    private void jTextFieldNumSimulationsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldNumSimulationsActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldNumSimulationsActionPerformed

    private void jComboBoxSpeedActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxSpeedActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jComboBoxSpeedActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(MainGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(MainGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(MainGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(MainGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /*
         * Create and display the form
         */
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new MainGUI().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonRunSimulation;
    private javax.swing.JButton jButtonSaveResults;
    private javax.swing.JComboBox jComboBoxSpeed;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabelCouple;
    private javax.swing.JLabel jLabelFamilyBabies;
    private javax.swing.JLabel jLabelFamilyWithoutBabies;
    private javax.swing.JLabel jLabelFriends;
    private javax.swing.JLabel jLabelInput1;
    private javax.swing.JLabel jLabelIterations;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTable jTable1;
    private javax.swing.JTable jTableResults;
    private javax.swing.JTable jTableSummary;
    private javax.swing.JTextField jTextFieldIterations;
    private javax.swing.JTextField jTextFieldJoker;
    private javax.swing.JTextField jTextFieldNumSimulations;
    private javax.swing.JTextField jTextFieldObstructive;
    private javax.swing.JTextField jTextFieldOccasional;
    private javax.swing.JTextField jTextFieldParticipant;
    private javax.swing.JTextField jTextFieldQuite;
    private javax.swing.JTextField jTextFieldTangent;
    // End of variables declaration//GEN-END:variables

   
}
